@@ -178,6 +178,8 @@
 #include <qmath.h>
 #include <private/qlayoutengine_p.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 using namespace QMdi;
@@ -465,9 +467,9 @@ void MinOverlapPlacer::getCandidatePlacements(const QSize &size, const QList<QRe
     }
 
     QList<int> xlist = xset.values();
-    qSort(xlist.begin(), xlist.end());
+    std::sort(xlist.begin(), xlist.end());
     QList<int> ylist = yset.values();
-    qSort(ylist.begin(), ylist.end());
+    std::sort(ylist.begin(), ylist.end());
 
     foreach (int y, ylist)
         foreach (int x, xlist)
@@ -2043,7 +2045,7 @@ void QMdiArea::setBackground(const QBrush &brush)
     if (d->background != brush) {
         d->background = brush;
         d->viewport->setAttribute(Qt::WA_OpaquePaintEvent, brush.isOpaque());
-        update();
+        d->viewport->update();
     }
 }
 
@@ -2088,7 +2090,7 @@ void QMdiArea::setOption(AreaOption option, bool on)
 }
 
 /*!
-    Returns true if \a option is enabled; otherwise returns false.
+    Returns \c true if \a option is enabled; otherwise returns \c false.
 
     \sa AreaOption, setOption()
 */
