@@ -43,16 +43,15 @@
 
 #include "qserialport_win_p.h"
 
+#include <QtCore/qcoreevent.h>
+
 #ifndef Q_OS_WINCE
 #include <QtCore/qelapsedtimer.h>
 #include <QtCore/qvector.h>
 #endif
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 #include <QtCore/qwineventnotifier.h>
-#else
-#include "qt4support/qwineventnotifier_p.h"
-#endif
+#include <algorithm>
 
 #ifndef CTL_CODE
 #  define CTL_CODE(DeviceType, Function, Method, Access) ( \
@@ -91,6 +90,7 @@ QT_BEGIN_NAMESPACE
 
 class AbstractOverlappedEventNotifier : public QWinEventNotifier
 {
+    Q_OBJECT
 public:
     enum Type { CommEvent, ReadCompletionEvent, WriteCompletionEvent };
 
@@ -106,7 +106,6 @@ public:
 
     virtual ~AbstractOverlappedEventNotifier() {
         setEnabled(false);
-        ::CancelIo(o.hEvent);
         ::CloseHandle(o.hEvent);
     }
 
@@ -116,7 +115,8 @@ public:
 protected:
     bool event(QEvent *e) Q_DECL_OVERRIDE {
         const bool ret = QWinEventNotifier::event(e);
-        processCompletionRoutine();
+        if (e->type() == QEvent::WinEventAct)
+            processCompletionRoutine();
         return ret;
     }
 
@@ -127,6 +127,7 @@ protected:
 
 class CommOverlappedEventNotifier : public AbstractOverlappedEventNotifier
 {
+    Q_OBJECT
 public:
     CommOverlappedEventNotifier(QSerialPortPrivate *d, DWORD eventMask, QObject *parent)
         : AbstractOverlappedEventNotifier(d, CommEvent, false, parent)
@@ -173,6 +174,7 @@ private:
 
 class ReadOverlappedCompletionNotifier : public AbstractOverlappedEventNotifier
 {
+    Q_OBJECT
 public:
     ReadOverlappedCompletionNotifier(QSerialPortPrivate *d, QObject *parent)
         : AbstractOverlappedEventNotifier(d, ReadCompletionEvent, false, parent) {}
@@ -180,24 +182,25 @@ public:
     bool processCompletionRoutine() Q_DECL_OVERRIDE {
         DWORD numberOfBytesTransferred = 0;
         ::GetOverlappedResult(dptr->descriptor, &o, &numberOfBytesTransferred, FALSE);
-        bool ret = dptr->completeAsyncRead(numberOfBytesTransferred);
+        dptr->completeAsyncRead(numberOfBytesTransferred);
 
         // start async read for possible remainder into driver queue
-        if (ret && (numberOfBytesTransferred > 0) && (dptr->policy == QSerialPort::IgnorePolicy)) {
+        if ((numberOfBytesTransferred > 0) && (dptr->policy == QSerialPort::IgnorePolicy)) {
             dptr->startAsyncRead();
         } else { // driver queue is emplty, so startup wait comm event
             CommOverlappedEventNotifier *n =
-                    reinterpret_cast<CommOverlappedEventNotifier *>(dptr->lookupCommEventNotifier());
+                    qobject_cast<CommOverlappedEventNotifier *>(dptr->lookupCommEventNotifier());
             if (n)
                 n->startWaitCommEvent();
         }
 
-        return ret;
+        return true;
     }
 };
 
 class WriteOverlappedCompletionNotifier : public AbstractOverlappedEventNotifier
 {
+    Q_OBJECT
 public:
     WriteOverlappedCompletionNotifier(QSerialPortPrivate *d, QObject *parent)
         : AbstractOverlappedEventNotifier(d, WriteCompletionEvent, false, parent) {}
@@ -206,10 +209,13 @@ public:
         setEnabled(false);
         DWORD numberOfBytesTransferred = 0;
         ::GetOverlappedResult(dptr->descriptor, &o, &numberOfBytesTransferred, FALSE);
-        return dptr->completeAsyncWrite(numberOfBytesTransferred);
+        dptr->completeAsyncWrite(numberOfBytesTransferred);
+        return true;
     }
 };
 
+#include "qserialport_win.moc"
+
 QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
     : QSerialPortPrivateData(q)
     , descriptor(INVALID_HANDLE_VALUE)
@@ -224,6 +230,8 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
 
 bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
 {
+    Q_Q(QSerialPort);
+
     DWORD desiredAccess = 0;
     DWORD originalEventMask = EV_ERR;
 
@@ -238,12 +246,12 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
                               desiredAccess, 0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
 
     if (descriptor == INVALID_HANDLE_VALUE) {
-        q_ptr->setError(decodeSystemError());
+        q->setError(decodeSystemError());
         return false;
     }
 
     if (!::GetCommState(descriptor, &restoredDcb)) {
-        q_ptr->setError(decodeSystemError());
+        q->setError(decodeSystemError());
         return false;
     }
 
@@ -259,7 +267,7 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
 
     if (!::GetCommTimeouts(descriptor, &restoredCommTimeouts)) {
-        q_ptr->setError(decodeSystemError());
+        q->setError(decodeSystemError());
         return false;
     }
 
@@ -270,11 +278,11 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
 
     if (originalEventMask & EV_RXCHAR) {
-        QWinEventNotifier *n = new ReadOverlappedCompletionNotifier(this, q_ptr);
+        QWinEventNotifier *n = new ReadOverlappedCompletionNotifier(this, q);
         n->setEnabled(true);
     }
 
-    QWinEventNotifier *n = new CommOverlappedEventNotifier(this, originalEventMask, q_ptr);
+    QWinEventNotifier *n = new CommOverlappedEventNotifier(this, originalEventMask, q);
     n->setEnabled(true);
 
     detectDefaultSettings();
@@ -310,16 +318,19 @@ void QSerialPortPrivate::close()
 
 #endif // #ifndef Q_OS_WINCE
 
-QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals() const
+QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
 {
+    Q_Q(QSerialPort);
+
     DWORD modemStat = 0;
-    QSerialPort::PinoutSignals ret = QSerialPort::NoSignal;
 
     if (!::GetCommModemStatus(descriptor, &modemStat)) {
-        q_ptr->setError(decodeSystemError());
-        return ret;
+        q->setError(decodeSystemError());
+        return QSerialPort::NoSignal;
     }
 
+    QSerialPort::PinoutSignals ret = QSerialPort::NoSignal;
+
     if (modemStat & MS_CTS_ON)
         ret |= QSerialPort::ClearToSendSignal;
     if (modemStat & MS_DSR_ON)
@@ -360,14 +371,14 @@ bool QSerialPortPrivate::flush()
     return startAsyncWrite() && ::FlushFileBuffers(descriptor);
 }
 
-bool QSerialPortPrivate::clear(QSerialPort::Directions dir)
+bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
 {
     DWORD flags = 0;
-    if (dir & QSerialPort::Input) {
+    if (directions & QSerialPort::Input) {
         flags |= PURGE_RXABORT | PURGE_RXCLEAR;
         actualReadBufferSize = 0;
     }
-    if (dir & QSerialPort::Output) {
+    if (directions & QSerialPort::Output) {
         flags |= PURGE_TXABORT | PURGE_TXCLEAR;
         actualWriteBufferSize = 0;
         acyncWritePosition = 0;
@@ -467,6 +478,8 @@ qint64 QSerialPortPrivate::writeToBuffer(const char *data, qint64 maxSize)
 
 bool QSerialPortPrivate::waitForReadyRead(int msecs)
 {
+    Q_Q(QSerialPort);
+
     QElapsedTimer stopWatch;
     stopWatch.start();
 
@@ -477,7 +490,7 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
         if (!waitAnyEvent(timeoutValue(msecs, stopWatch.elapsed()), &timedOut, &n) || !n) {
             // This is occur timeout or another error
             if (!timedOut)
-                q_ptr->setError(decodeSystemError());
+                q->setError(decodeSystemError());
             return false;
         }
 
@@ -501,6 +514,8 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
 
 bool QSerialPortPrivate::waitForBytesWritten(int msecs)
 {
+    Q_Q(QSerialPort);
+
     if (writeBuffer.isEmpty())
         return false;
 
@@ -513,7 +528,7 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
 
         if (!waitAnyEvent(timeoutValue(msecs, stopWatch.elapsed()), &timedOut, &n) || !n) {
             if (!timedOut)
-                q_ptr->setError(decodeSystemError());
+                q->setError(decodeSystemError());
             return false;
         }
 
@@ -535,10 +550,12 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
 
 #endif // #ifndef Q_OS_WINCE
 
-bool QSerialPortPrivate::setBaudRate(qint32 baudRate, QSerialPort::Directions dir)
+bool QSerialPortPrivate::setBaudRate(qint32 baudRate, QSerialPort::Directions directions)
 {
-    if (dir != QSerialPort::AllDirections) {
-        q_ptr->setError(QSerialPort::UnsupportedOperationError);
+    Q_Q(QSerialPort);
+
+    if (directions != QSerialPort::AllDirections) {
+        q->setError(QSerialPort::UnsupportedOperationError);
         return false;
     }
     currentDcb.BaudRate = baudRate;
@@ -631,6 +648,8 @@ bool QSerialPortPrivate::setDataErrorPolicy(QSerialPort::DataErrorPolicy policy)
 
 bool QSerialPortPrivate::startAsyncRead()
 {
+    Q_Q(QSerialPort);
+
     DWORD bytesToRead = policy == QSerialPort::IgnorePolicy ? ReadChunkSize : 1;
 
     if (readBufferMaxSize && bytesToRead > (readBufferMaxSize - readBuffer.size())) {
@@ -644,7 +663,7 @@ bool QSerialPortPrivate::startAsyncRead()
 
     AbstractOverlappedEventNotifier *n = lookupReadCompletionNotifier();
     if (!n) {
-        q_ptr->setError(QSerialPort::ResourceError);
+        q->setError(QSerialPort::ResourceError);
         return false;
     }
 
@@ -657,7 +676,7 @@ bool QSerialPortPrivate::startAsyncRead()
     if (error != QSerialPort::NoError) {
         if (error != QSerialPort::ResourceError)
             error = QSerialPort::ReadError;
-        q_ptr->setError(error);
+        q->setError(error);
 
         readBuffer.truncate(actualReadBufferSize);
         return false;
@@ -668,6 +687,8 @@ bool QSerialPortPrivate::startAsyncRead()
 
 bool QSerialPortPrivate::startAsyncWrite(int maxSize)
 {
+    Q_Q(QSerialPort);
+
     qint64 nextSize = 0;
     const char *ptr = writeBuffer.readPointerAtPosition(acyncWritePosition, nextSize);
 
@@ -682,7 +703,7 @@ bool QSerialPortPrivate::startAsyncWrite(int maxSize)
 
     AbstractOverlappedEventNotifier *n = lookupFreeWriteCompletionNotifier();
     if (!n) {
-        q_ptr->setError(QSerialPort::ResourceError);
+        q->setError(QSerialPort::ResourceError);
         return false;
     }
 
@@ -698,7 +719,7 @@ bool QSerialPortPrivate::startAsyncWrite(int maxSize)
          if (error != QSerialPort::ResourceError)
              error = QSerialPort::WriteError;
 
-         q_ptr->setError(error);
+         q->setError(error);
          return false;
     }
 
@@ -707,34 +728,36 @@ bool QSerialPortPrivate::startAsyncWrite(int maxSize)
 
 #endif // #ifndef Q_OS_WINCE
 
-bool QSerialPortPrivate::processIoErrors(bool error)
+void QSerialPortPrivate::processIoErrors(bool error)
 {
+    Q_Q(QSerialPort);
+
     if (error) {
-        q_ptr->setError(QSerialPort::ResourceError);
-        return true;
+        q->setError(QSerialPort::ResourceError);
+        return;
     }
 
     DWORD errors = 0;
-    const bool ret = ::ClearCommError(descriptor, &errors, NULL);
-    if (ret && errors) {
+    if (::ClearCommError(descriptor, &errors, NULL) && errors) {
         if (errors & CE_FRAME) {
-            q_ptr->setError(QSerialPort::FramingError);
+            q->setError(QSerialPort::FramingError);
         } else if (errors & CE_RXPARITY) {
-            q_ptr->setError(QSerialPort::ParityError);
+            q->setError(QSerialPort::ParityError);
             parityErrorOccurred = true;
         } else if (errors & CE_BREAK) {
-            q_ptr->setError(QSerialPort::BreakConditionError);
+            q->setError(QSerialPort::BreakConditionError);
         } else {
-            q_ptr->setError(QSerialPort::UnknownError);
+            q->setError(QSerialPort::UnknownError);
         }
     }
-    return ret;
 }
 
 #ifndef Q_OS_WINCE
 
-bool QSerialPortPrivate::completeAsyncRead(DWORD numberOfBytes)
+void QSerialPortPrivate::completeAsyncRead(DWORD numberOfBytes)
 {
+    Q_Q(QSerialPort);
+
     actualReadBufferSize += qint64(numberOfBytes);
     readBuffer.truncate(actualReadBufferSize);
 
@@ -749,14 +772,14 @@ bool QSerialPortPrivate::completeAsyncRead(DWORD numberOfBytes)
             if (policy == QSerialPort::SkipPolicy) {
                 readBuffer.getChar();
                 // Force returning without emitting a readyRead() signal
-                return true;
+                return;
             }
 
             // Abort receiving
             if (policy == QSerialPort::StopReceivingPolicy) {
                 readyReadEmitted = true;
-                emit q_ptr->readyRead();
-                return true;
+                emit q->readyRead();
+                return;
             }
 
             // Replace received character by zero
@@ -768,30 +791,31 @@ bool QSerialPortPrivate::completeAsyncRead(DWORD numberOfBytes)
         }
 
         readyReadEmitted = true;
-        emit q_ptr->readyRead();
+        emit q->readyRead();
     }
-    return true;
 }
 
-bool QSerialPortPrivate::completeAsyncWrite(DWORD numberOfBytes)
+void QSerialPortPrivate::completeAsyncWrite(DWORD numberOfBytes)
 {
+    Q_Q(QSerialPort);
+
     writeBuffer.free(numberOfBytes);
     actualWriteBufferSize -= qint64(numberOfBytes);
     acyncWritePosition -= qint64(numberOfBytes);
 
     if (numberOfBytes > 0)
-        emit q_ptr->bytesWritten(numberOfBytes);
+        emit q->bytesWritten(numberOfBytes);
 
     if (writeBuffer.isEmpty())
         writeSequenceStarted = false;
     else
         startAsyncWrite(WriteChunkSize);
-
-    return true;
 }
 
 AbstractOverlappedEventNotifier *QSerialPortPrivate::lookupFreeWriteCompletionNotifier()
 {
+    Q_Q(QSerialPort);
+
     // find first free not running write notifier
     foreach (AbstractOverlappedEventNotifier *n, notifiers) {
         if ((n->type() == AbstractOverlappedEventNotifier::WriteCompletionEvent)
@@ -800,7 +824,7 @@ AbstractOverlappedEventNotifier *QSerialPortPrivate::lookupFreeWriteCompletionNo
         }
     }
     // if all write notifiers in use, then create new write notifier
-    return new WriteOverlappedCompletionNotifier(this, q_ptr);
+    return new WriteOverlappedCompletionNotifier(this, q);
 }
 
 AbstractOverlappedEventNotifier *QSerialPortPrivate::lookupCommEventNotifier()
@@ -823,8 +847,10 @@ AbstractOverlappedEventNotifier *QSerialPortPrivate::lookupReadCompletionNotifie
 
 bool QSerialPortPrivate::updateDcb()
 {
+    Q_Q(QSerialPort);
+
     if (!::SetCommState(descriptor, &currentDcb)) {
-        q_ptr->setError(decodeSystemError());
+        q->setError(decodeSystemError());
         return false;
     }
     return true;
@@ -832,8 +858,10 @@ bool QSerialPortPrivate::updateDcb()
 
 bool QSerialPortPrivate::updateCommTimeouts()
 {
+    Q_Q(QSerialPort);
+
     if (!::SetCommTimeouts(descriptor, &currentCommTimeouts)) {
-        q_ptr->setError(decodeSystemError());
+        q->setError(decodeSystemError());
         return false;
     }
     return true;
@@ -862,7 +890,8 @@ void QSerialPortPrivate::detectDefaultSettings()
         dataBits = QSerialPort::Data8;
         break;
     default:
-        dataBits = QSerialPort::UnknownDataBits;
+        qWarning("%s: Unexpected data bits settings", Q_FUNC_INFO);
+        dataBits = QSerialPort::Data8;
         break;
     }
 
@@ -877,8 +906,10 @@ void QSerialPortPrivate::detectDefaultSettings()
         parity = QSerialPort::EvenParity;
     else if ((currentDcb.Parity == ODDPARITY) && currentDcb.fParity)
         parity = QSerialPort::OddParity;
-    else
-        parity = QSerialPort::UnknownParity;
+    else {
+        qWarning("%s: Unexpected parity settings", Q_FUNC_INFO);
+        parity = QSerialPort::NoParity;
+    }
 
     // Detect stopbits.
     switch (currentDcb.StopBits) {
@@ -892,7 +923,8 @@ void QSerialPortPrivate::detectDefaultSettings()
         stopBits = QSerialPort::TwoStop;
         break;
     default:
-        stopBits = QSerialPort::UnknownStopBits;
+        qWarning("%s: Unexpected stop bits settings", Q_FUNC_INFO);
+        stopBits = QSerialPort::OneStop;
         break;
     }
 
@@ -906,8 +938,10 @@ void QSerialPortPrivate::detectDefaultSettings()
     } else if (currentDcb.fOutxCtsFlow && (currentDcb.fRtsControl == RTS_CONTROL_HANDSHAKE)
                && !currentDcb.fInX && !currentDcb.fOutX) {
         flow = QSerialPort::HardwareControl;
-    } else
-        flow = QSerialPort::UnknownFlowControl;
+    } else {
+        qWarning("%s: Unexpected flow control settings", Q_FUNC_INFO);
+        flow = QSerialPort::NoFlowControl;
+    }
 }
 
 QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
@@ -950,15 +984,18 @@ QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
 bool QSerialPortPrivate::waitAnyEvent(int msecs, bool *timedOut,
                                      AbstractOverlappedEventNotifier **triggeredNotifier)
 {
+    Q_Q(QSerialPort);
+
     Q_ASSERT(timedOut);
 
     QVector<HANDLE> handles =  notifiers.keys().toVector();
     DWORD waitResult = ::WaitForMultipleObjects(handles.count(),
                                                 handles.constData(),
                                                 FALSE, // wait any event
-                                                qMax(msecs, 0));
+                                                msecs == -1 ? INFINITE : msecs);
     if (waitResult == WAIT_TIMEOUT) {
         *timedOut = true;
+        q->setError(QSerialPort::TimeoutError);
         return false;
     }
 
@@ -970,7 +1007,7 @@ bool QSerialPortPrivate::waitAnyEvent(int msecs, bool *timedOut,
     return true;
 }
 
-static const QLatin1String defaultPathPrefix("\\\\.\\");
+static const QString defaultPathPrefix = QStringLiteral("\\\\.\\");
 
 QString QSerialPortPrivate::portNameToSystemLocation(const QString &port)
 {
@@ -1065,7 +1102,8 @@ static const QList<qint32> standardBaudRatePairList()
 qint32 QSerialPortPrivate::baudRateFromSetting(qint32 setting)
 {
     const QList<qint32> baudRatePairs = standardBaudRatePairList();
-    const QList<qint32>::const_iterator baudRatePairListConstIterator = qFind(baudRatePairs, setting);
+    const QList<qint32>::const_iterator baudRatePairListConstIterator
+            = std::find(baudRatePairs.constBegin(), baudRatePairs.constEnd(), setting);
 
     return (baudRatePairListConstIterator != baudRatePairs.constEnd()) ? *baudRatePairListConstIterator : 0;
 }
@@ -1073,7 +1111,8 @@ qint32 QSerialPortPrivate::baudRateFromSetting(qint32 setting)
 qint32 QSerialPortPrivate::settingFromBaudRate(qint32 baudRate)
 {
     const QList<qint32> baudRatePairList = standardBaudRatePairList();
-    const QList<qint32>::const_iterator baudRatePairListConstIterator = qFind(baudRatePairList, baudRate);
+    const QList<qint32>::const_iterator baudRatePairListConstIterator
+            = std::find(baudRatePairList.constBegin(), baudRatePairList.constEnd(), baudRate);
 
     return (baudRatePairListConstIterator != baudRatePairList.constEnd()) ? *baudRatePairListConstIterator : 0;
 }
@@ -1090,4 +1129,10 @@ QList<qint32> QSerialPortPrivate::standardBaudRates()
     return ret;
 }
 
+QSerialPort::Handle QSerialPort::handle() const
+{
+    Q_D(const QSerialPort);
+    return d->descriptor;
+}
+
 QT_END_NAMESPACE
